@@ -7,7 +7,6 @@ import tidy3d as td
 from tidy3d.constants import fp_eps
 
 from tidy3d.components.grid.mesher import GradedMesher
-from ..utils import assert_log_level
 
 np.random.seed(4)
 
@@ -651,7 +650,7 @@ def test_mesher_timeout():
     grid = sim.grid
 
 
-def test_shapely_strtree_warnings(caplog):
+def test_shapely_strtree_warnings():
 
     with warnings.catch_warnings():
         warnings.simplefilter("error")
