@@ -5,10 +5,10 @@ import matplotlib.pylab as plt
 
 import numpy as np
 import tidy3d as td
-from tidy3d.log import SetupError, ValidationError, Tidy3dKeyError
+from tidy3d.exceptions import SetupError, ValidationError, Tidy3dKeyError
 from tidy3d.components import simulation
 from tidy3d.components.simulation import MAX_NUM_MEDIUMS
-from ..utils import assert_log_level, SIM_FULL
+from ..utils import assert_log_level, SIM_FULL, log_capture
 
 SIM = td.Simulation(size=(1, 1, 1), run_time=1e-12, grid_spec=td.GridSpec(wavelength=1.0))
 
@@ -91,6 +91,15 @@ def test_sim_init():
     sim.epsilon(m)
 
 
+# TODO: remove for 2.0
+def test_deprecation_defaults(log_capture):
+    """Make sure deprecation warnings thrown if defaults used."""
+    s = td.Simulation(
+        size=(1, 1, 1), run_time=1e-12, grid_spec=td.GridSpec.uniform(dl=0.1), boundary_spec=None
+    )
+    assert_log_level(log_capture, "warning")
+
+
 def test_sim_bounds():
     """make sure bounds are working correctly"""
 
@@ -111,6 +120,7 @@ def test_sim_bounds():
                     geometry=td.Box(size=(1, 1, 1), center=shifted_center), medium=td.Medium()
                 )
             ],
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
     # create all permutations of squares being shifted 1, -1, or zero in all three directions
@@ -141,11 +151,20 @@ def test_sim_size():
     grid_spec = td.GridSpec(grid_x=mesh1d, grid_y=mesh1d, grid_z=mesh1d)
 
     with pytest.raises(SetupError):
-        s = td.Simulation(size=(1, 1, 1), grid_spec=grid_spec, run_time=1e-12)
+        s = td.Simulation(
+            size=(1, 1, 1),
+            grid_spec=grid_spec,
+            run_time=1e-12,
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
+        )
         s._validate_size()
 
     with pytest.raises(SetupError):
-        s = td.Simulation(size=(1, 1, 1), run_time=1e-7)
+        s = td.Simulation(
+            size=(1, 1, 1),
+            run_time=1e-7,
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
+        )
         s._validate_size()
 
 
@@ -161,12 +180,13 @@ def _test_monitor_size():
                 )
             ],
             run_time=1e-12,
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
         s.validate_pre_upload()
 
 
-@pytest.mark.parametrize("freq, log_level", [(1.5, 30), (2.5, None), (3.5, 30)])
-def test_monitor_medium_frequency_range(caplog, freq, log_level):
+@pytest.mark.parametrize("freq, log_level", [(1.5, "warning"), (2.5, None), (3.5, "warning")])
+def test_monitor_medium_frequency_range(log_capture, freq, log_level):
     # monitor frequency above or below a given medium's range should throw a warning
 
     size = (1, 1, 1)
@@ -184,12 +204,13 @@ def test_monitor_medium_frequency_range(caplog, freq, log_level):
         monitors=[mnt],
         sources=[src],
         run_time=1e-12,
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
-    assert_log_level(caplog, log_level)
+    assert_log_level(log_capture, log_level)
 
 
-@pytest.mark.parametrize("fwidth, log_level", [(0.1, 30), (2, None)])
-def test_monitor_simulation_frequency_range(caplog, fwidth, log_level):
+@pytest.mark.parametrize("fwidth, log_level", [(0.1, "warning"), (2, None)])
+def test_monitor_simulation_frequency_range(log_capture, fwidth, log_level):
     # monitor frequency outside of the simulation's frequency range should throw a warning
 
     size = (1, 1, 1)
@@ -199,8 +220,14 @@ def test_monitor_simulation_frequency_range(caplog, fwidth, log_level):
         polarization="Ex",
     )
     mnt = td.FieldMonitor(size=(0, 0, 0), name="freq", freqs=[1.5])
-    sim = td.Simulation(size=(1, 1, 1), monitors=[mnt], sources=[src], run_time=1e-12)
-    assert_log_level(caplog, log_level)
+    sim = td.Simulation(
+        size=(1, 1, 1),
+        monitors=[mnt],
+        sources=[src],
+        run_time=1e-12,
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
+    )
+    assert_log_level(log_capture, log_level)
 
 
 def test_validate_bloch_with_symmetry():
@@ -218,7 +245,7 @@ def test_validate_bloch_with_symmetry():
         )
 
 
-def test_validate_plane_wave_boundaries(caplog):
+def test_validate_plane_wave_boundaries(log_capture):
     src1 = td.PlaneWave(
         source_time=td.GaussianPulse(freq0=2.5e14, fwidth=1e13),
         center=(0, 0, 0),
@@ -284,7 +311,7 @@ def test_validate_plane_wave_boundaries(caplog):
         sources=[src2],
         boundary_spec=bspec3,
     )
-    assert_log_level(caplog, 30)
+    assert_log_level(log_capture, "warning")
 
     # angled incidence plane wave with wrong Bloch vector should warn
     td.Simulation(
@@ -293,7 +320,43 @@ def test_validate_plane_wave_boundaries(caplog):
         sources=[src2],
         boundary_spec=bspec4,
     )
-    assert_log_level(caplog, 30)
+    assert_log_level(log_capture, "warning")
+
+
+def test_validate_zero_dim_boundaries(log_capture):
+
+    # zero-dim simulation with an absorbing boundary in that direction should warn
+    src = td.PlaneWave(
+        source_time=td.GaussianPulse(freq0=2.5e14, fwidth=1e13),
+        center=(0, 0, 0),
+        size=(td.inf, 0, td.inf),
+        direction="+",
+        pol_angle=0.0,
+    )
+
+    td.Simulation(
+        size=(1, 1, 0),
+        run_time=1e-12,
+        sources=[src],
+        boundary_spec=td.BoundarySpec(
+            x=td.Boundary.periodic(),
+            y=td.Boundary.periodic(),
+            z=td.Boundary.pml(),
+        ),
+    )
+    assert_log_level(log_capture, "warning")
+
+    # zero-dim simulation with an absorbing boundary any other direction should not warn
+    td.Simulation(
+        size=(1, 1, 0),
+        run_time=1e-12,
+        sources=[src],
+        boundary_spec=td.BoundarySpec(
+            x=td.Boundary.pml(),
+            y=td.Boundary.stable_pml(),
+            z=td.Boundary.pec(),
+        ),
+    )
 
 
 def test_validate_components_none():
@@ -434,15 +497,16 @@ def test_min_sym_box():
     S.min_sym_box(b)
 
 
-def test_discretize_non_intersect(caplog):
+def test_discretize_non_intersect(log_capture):
     SIM.discretize(box=td.Box(center=(-20, -20, -20), size=(1, 1, 1)))
-    assert_log_level(caplog, 40)
+    assert_log_level(log_capture, "error")
 
 
 def test_filter_structures():
     s1 = td.Structure(geometry=td.Box(size=(1, 1, 1)), medium=SIM.medium)
     s2 = td.Structure(geometry=td.Box(size=(1, 1, 1), center=(1, 1, 1)), medium=SIM.medium)
-    SIM._filter_structures_plane(structures=[s1, s2], z=1.5)
+    plane = td.Box(center=(0, 0, 1.5), size=(td.inf, td.inf, 0))
+    SIM._filter_structures_plane(structures=[s1, s2], plane=plane)
 
 
 def test_get_structure_plot_params():
@@ -458,7 +522,7 @@ def test_get_structure_plot_params():
     assert pp.facecolor == "gold"
 
 
-def test_warn_sim_background_medium_freq_range(caplog):
+def test_warn_sim_background_medium_freq_range(log_capture):
     S = SIM.copy(
         update=dict(
             sources=(
@@ -470,11 +534,11 @@ def test_warn_sim_background_medium_freq_range(caplog):
             medium=td.Medium(frequency_range=(0, 1)),
         )
     )
-    assert_log_level(caplog, 30)
+    assert_log_level(log_capture, "warning")
 
 
-@pytest.mark.parametrize("grid_size,log_level", [(0.001, None), (3, 30)])
-def test_large_grid_size(caplog, grid_size, log_level):
+@pytest.mark.parametrize("grid_size,log_level", [(0.001, None), (3, "warning")])
+def test_large_grid_size(log_capture, grid_size, log_level):
     # small fwidth should be inside range, large one should throw warning
 
     medium = td.Medium(permittivity=2, frequency_range=(2e14, 3e14))
@@ -489,13 +553,14 @@ def test_large_grid_size(caplog, grid_size, log_level):
         structures=[box],
         sources=[src],
         run_time=1e-12,
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
 
-    assert_log_level(caplog, log_level)
+    assert_log_level(log_capture, log_level)
 
 
-@pytest.mark.parametrize("box_size,log_level", [(0.001, None), (9.9, 30), (20, None)])
-def test_sim_structure_gap(caplog, box_size, log_level):
+@pytest.mark.parametrize("box_size,log_level", [(0.001, None), (9.9, "warning"), (20, None)])
+def test_sim_structure_gap(log_capture, box_size, log_level):
     """Make sure the gap between a structure and PML is not too small compared to lambda0."""
     medium = td.Medium(permittivity=2)
     box = td.Structure(geometry=td.Box(size=(box_size, box_size, box_size)), medium=medium)
@@ -515,7 +580,7 @@ def test_sim_structure_gap(caplog, box_size, log_level):
         ),
         run_time=1e-12,
     )
-    assert_log_level(caplog, log_level)
+    assert_log_level(log_capture, log_level)
 
 
 def test_sim_plane_wave_error():
@@ -543,6 +608,7 @@ def test_sim_plane_wave_error():
         structures=[box_transparent],
         sources=[src],
         run_time=1e-12,
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
 
     # with non-transparent box, raise
@@ -552,6 +618,7 @@ def test_sim_plane_wave_error():
             medium=medium_bg,
             structures=[box_transparent, box],
             sources=[src],
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
 
@@ -563,9 +630,9 @@ def test_sim_monitor_homogeneous():
     medium_bg = td.Medium(permittivity=2)
     medium_air = td.Medium(permittivity=1)
 
-    box = td.Structure(geometry=td.Box(size=(0.1, 0.1, 0.1)), medium=medium_air)
+    box = td.Structure(geometry=td.Box(size=(0.2, 0.1, 0.1)), medium=medium_air)
 
-    box_transparent = td.Structure(geometry=td.Box(size=(0.1, 0.1, 0.1)), medium=medium_bg)
+    box_transparent = td.Structure(geometry=td.Box(size=(0.2, 0.1, 0.1)), medium=medium_bg)
 
     monitor_n2f = td.FieldProjectionAngleMonitor(
         center=(0, 0, 0),
@@ -593,16 +660,13 @@ def test_sim_monitor_homogeneous():
         normal_dir="+",
     )
 
-    src = td.PlaneWave(
+    src = td.PointDipole(
         source_time=td.GaussianPulse(freq0=2.5e14, fwidth=1e13),
         center=(0, 0, 0),
-        size=(td.inf, td.inf, 0),
-        direction="+",
-        pol_angle=-1.0,
+        polarization="Ex",
     )
 
-    for monitor in [monitor_n2f, monitor_n2f_vol, monitor_diffraction]:
-
+    for monitor in [monitor_n2f_vol]:
         # with transparent box continue
         sim1 = td.Simulation(
             size=(1, 1, 1),
@@ -611,6 +675,7 @@ def test_sim_monitor_homogeneous():
             sources=[src],
             run_time=1e-12,
             monitors=[monitor],
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
         # with non-transparent box, raise
@@ -618,13 +683,16 @@ def test_sim_monitor_homogeneous():
             _ = td.Simulation(
                 size=(1, 1, 1),
                 medium=medium_bg,
-                structures=[box_transparent, box],
+                structures=[box],
                 sources=[src],
                 monitors=[monitor],
                 run_time=1e-12,
+                boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
             )
 
-    mediums = td.Simulation.intersecting_media(monitor_n2f_vol, sim1.structures)
+    mediums = td.Simulation.intersecting_media(monitor_n2f_vol, [box])
+    assert len(mediums) == 1
+    mediums = td.Simulation.intersecting_media(monitor_n2f_vol, [box_transparent])
     assert len(mediums) == 1
 
     # when another medium intersects an excluded surface, no errors should be raised
@@ -638,14 +706,6 @@ def test_sim_monitor_homogeneous():
         exclude_surfaces=["x-", "z-"],
     )
 
-    src = td.PlaneWave(
-        source_time=td.GaussianPulse(freq0=2.5e14, fwidth=1e13),
-        center=(0, 0, -0.5),
-        size=(td.inf, td.inf, 0),
-        direction="+",
-        pol_angle=-1.0,
-    )
-
     _ = td.Simulation(
         size=(1, 1, 1),
         medium=medium_bg,
@@ -653,10 +713,11 @@ def test_sim_monitor_homogeneous():
         sources=[src],
         monitors=[monitor_n2f_vol_exclude],
         run_time=1e-12,
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
 
 
-def test_proj_monitor_distance(caplog):
+def test_proj_monitor_distance(log_capture):
     """Make sure a warning is issued if the projection distance for exact projections
     is very large compared to the simulation domain size.
     """
@@ -709,8 +770,9 @@ def test_proj_monitor_distance(caplog):
         sources=[src],
         run_time=1e-12,
         monitors=[monitor_n2f_far],
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
-    assert_log_level(caplog, 30)
+    assert_log_level(log_capture, "warning")
 
     # proj_distance not too large - don't warn
     _ = td.Simulation(
@@ -719,6 +781,7 @@ def test_proj_monitor_distance(caplog):
         sources=[src],
         run_time=1e-12,
         monitors=[monitor_n2f],
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
 
     # proj_distance large but using approximations - don't warn
@@ -728,6 +791,7 @@ def test_proj_monitor_distance(caplog):
         sources=[src],
         run_time=1e-12,
         monitors=[monitor_n2f_approx],
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
 
 
@@ -763,6 +827,7 @@ def test_diffraction_medium():
             sources=[src],
             run_time=1e-12,
             monitors=[monitor],
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
     with pytest.raises(SetupError):
@@ -772,14 +837,20 @@ def test_diffraction_medium():
             sources=[src],
             monitors=[monitor],
             run_time=1e-12,
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
 
 @pytest.mark.parametrize(
     "box_size,log_level",
-    [((0.1, 0.1, 0.1), None), ((1, 0.1, 0.1), 30), ((0.1, 1, 0.1), 30), ((0.1, 0.1, 1), 30)],
+    [
+        ((0.1, 0.1, 0.1), None),
+        ((1, 0.1, 0.1), "warning"),
+        ((0.1, 1, 0.1), "warning"),
+        ((0.1, 0.1, 1), "warning"),
+    ],
 )
-def test_sim_structure_extent(caplog, box_size, log_level):
+def test_sim_structure_extent(log_capture, box_size, log_level):
     """Make sure we warn if structure extends exactly to simulation edges."""
 
     src = td.UniformCurrentSource(
@@ -788,9 +859,15 @@ def test_sim_structure_extent(caplog, box_size, log_level):
         polarization="Ex",
     )
     box = td.Structure(geometry=td.Box(size=box_size), medium=td.Medium(permittivity=2))
-    sim = td.Simulation(size=(1, 1, 1), structures=[box], sources=[src], run_time=1e-12)
+    sim = td.Simulation(
+        size=(1, 1, 1),
+        structures=[box],
+        sources=[src],
+        run_time=1e-12,
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
+    )
 
-    assert_log_level(caplog, log_level)
+    assert_log_level(log_capture, log_level)
 
 
 def test_num_mediums():
@@ -802,7 +879,13 @@ def test_num_mediums():
         structures.append(
             td.Structure(geometry=td.Box(size=(1, 1, 1)), medium=td.Medium(permittivity=i + 1))
         )
-    sim = td.Simulation(size=(5, 5, 5), grid_spec=grid_spec, structures=structures, run_time=1e-12)
+    sim = td.Simulation(
+        size=(5, 5, 5),
+        grid_spec=grid_spec,
+        structures=structures,
+        run_time=1e-12,
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
+    )
 
     with pytest.raises(SetupError):
         structures.append(
@@ -861,6 +944,7 @@ def _test_names_default():
             td.FluxMonitor(size=(0, 1, 1), center=(0, -0.5, 0), freqs=[1], name="mon2"),
             td.FluxMonitor(size=(1, 0, 1), center=(0, -0.5, 0), freqs=[1], name="mon3"),
         ],
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
 
     for i, structure in enumerate(sim.structures):
@@ -888,6 +972,7 @@ def test_names_unique():
                     name="struct1",
                 ),
             ],
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
     with pytest.raises(SetupError) as e:
@@ -910,6 +995,7 @@ def test_names_unique():
                     name="source1",
                 ),
             ],
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
     with pytest.raises(SetupError) as e:
@@ -920,6 +1006,7 @@ def test_names_unique():
                 td.FluxMonitor(size=(1, 1, 0), center=(0, -0.5, 0), freqs=[1], name="mon1"),
                 td.FluxMonitor(size=(0, 1, 1), center=(0, -0.5, 0), freqs=[1], name="mon1"),
             ],
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
 
@@ -936,6 +1023,7 @@ def test_mode_object_syms():
             run_time=1e-12,
             symmetry=(1, -1, 0),
             sources=[td.ModeSource(size=(2, 2, 0), direction="+", source_time=g)],
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
     # wrong mode monitor
@@ -949,6 +1037,7 @@ def test_mode_object_syms():
             monitors=[
                 td.ModeMonitor(size=(2, 2, 0), name="mnt", freqs=[2], mode_spec=td.ModeSpec())
             ],
+            boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
         )
 
     # right mode source (centered on the symmetry)
@@ -959,6 +1048,7 @@ def test_mode_object_syms():
         run_time=1e-12,
         symmetry=(1, -1, 0),
         sources=[td.ModeSource(center=(1, -1, 1), size=(2, 2, 0), direction="+", source_time=g)],
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
 
     # right mode monitor (entirely in the main quadrant)
@@ -973,4 +1063,38 @@ def test_mode_object_syms():
                 center=(2, 0, 1), size=(2, 2, 0), name="mnt", freqs=[2], mode_spec=td.ModeSpec()
             )
         ],
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
     )
+
+
+@pytest.mark.parametrize(
+    "size, num_struct, log_level", [(1, 1, None), (50, 1, "warning"), (1, 11000, "warning")]
+)
+def test_warn_large_epsilon(log_capture, size, num_struct, log_level):
+    """Make sure we get a warning if the epsilon grid is too large."""
+
+    structures = [
+        td.Structure(
+            geometry=td.Box(center=(0, 0, 0), size=(0.1, 0.1, 0.1)),
+            medium=td.Medium(permittivity=1.0),
+        )
+        for _ in range(num_struct)
+    ]
+
+    sim = td.Simulation(
+        size=(size, size, size),
+        grid_spec=td.GridSpec.uniform(dl=0.1),
+        run_time=1e-12,
+        boundary_spec=td.BoundarySpec.all_sides(boundary=td.Periodic()),
+        sources=[
+            td.ModeSource(
+                center=(0, 0, 0),
+                size=(td.inf, td.inf, 0),
+                direction="+",
+                source_time=td.GaussianPulse(freq0=1, fwidth=0.1),
+            )
+        ],
+        structures=structures,
+    )
+    sim.epsilon(box=td.Box(size=(size, size, size)))
+    assert_log_level(log_capture, log_level)
