@@ -1,9 +1,14 @@
 import os
 from pathlib import Path
+from typing import Dict
 
+import pytest
 import numpy as np
 from tidy3d import *
 import tidy3d as td
+from tidy3d.log import _get_level_int
+from tidy3d.web import BatchData
+
 
 """ utilities shared between all tests """
 
@@ -53,11 +58,12 @@ SIM_MONITORS = Simulation(
             name="mode",
         ),
     ],
+    boundary_spec=BoundarySpec.all_sides(boundary=Periodic()),
 )
 
 SIM_FULL = Simulation(
-    size=(10.0, 10.0, 10.0),
-    run_time=40e-11,
+    size=(8.0, 8.0, 8.0),
+    run_time=1e-12,
     structures=[
         Structure(
             geometry=Box(size=(1, 1, 1), center=(-1, 0, 0)),
@@ -122,7 +128,7 @@ SIM_FULL = Simulation(
         ),
         ModeSource(
             center=(0, 0.5, 0),
-            size=(inf, 0, inf),
+            size=(2, 0, 2),
             mode_spec=td.ModeSpec(),
             source_time=GaussianPulse(
                 freq0=2e14,
@@ -161,7 +167,7 @@ SIM_FULL = Simulation(
             waist_distances=(3.0, 4.0),
         ),
         CustomFieldSource(
-            center=(1, 2, 3),
+            center=(0, 1, 2),
             size=(2, 2, 0),
             source_time=GaussianPulse(
                 freq0=2e14,
@@ -184,10 +190,10 @@ SIM_FULL = Simulation(
         FieldMonitor(
             size=(0, 0, 0), center=(0, 0, 0), fields=["Ex"], freqs=[1.5e14, 2e14], name="field"
         ),
-        FieldTimeMonitor(size=(0, 0, 0), center=(0, 0, 0), name="field_time"),
+        FieldTimeMonitor(size=(0, 0, 0), center=(0, 0, 0), name="field_time", interval=100),
         FluxMonitor(size=(1, 1, 0), center=(0, 0, 0), freqs=[2e14, 2.5e14], name="flux"),
         FluxTimeMonitor(size=(1, 1, 0), center=(0, 0, 0), name="flux_time"),
-        PermittivityMonitor(size=(1, 1, 1), name="eps", freqs=[1e13]),
+        PermittivityMonitor(size=(1, 1, 0.1), name="eps", freqs=[1e14]),
         ModeMonitor(
             size=(1, 1, 0),
             center=(0, 0, 0),
@@ -255,13 +261,13 @@ SIM_FULL = Simulation(
         y=Boundary.bloch(bloch_vec=1),
         z=Boundary.periodic(),
     ),
-    shutoff=1e-6,
+    shutoff=1e-4,
     courant=0.8,
     subpixel=False,
     grid_spec=GridSpec(
         grid_x=AutoGrid(),
-        grid_y=CustomGrid(dl=101 * [0.01]),
-        grid_z=UniformGrid(dl=0.01),
+        grid_y=CustomGrid(dl=100 * [0.04]),
+        grid_z=UniformGrid(dl=0.05),
         override_structures=[
             td.Structure(
                 geometry=Box(size=(1, 1, 1), center=(-1, 0, 0)),
@@ -271,46 +277,20 @@ SIM_FULL = Simulation(
     ),
 )
 
-# Initialize simulation
-SIM_CONVERT = td.Simulation(
-    size=[4, 4, 4],
-    structures=[
-        td.Structure(
-            geometry=td.Box(center=[0, 0, 0], size=[1.5, 1.5, 1.5]),
-            medium=td.Medium.from_nk(n=2, k=0, freq=3e14),
-        )
-    ],
-    sources=[
-        td.UniformCurrentSource(
-            center=(0, -1.5, 0),
-            size=(0.4, 0.4, 0.4),
-            source_time=td.GaussianPulse(freq0=3e14, fwidth=1e13),
-            polarization="Ex",
-        ),
-    ],
-    monitors=[
-        td.FieldMonitor(
-            fields=["Ex", "Hy"],
-            center=(0, 0, 0),
-            size=(inf, 0, inf),
-            freqs=[3e14],
-            name="field_monitor",
-        )
-    ],
-    run_time=1e-12,
-    boundary_spec=BoundarySpec.pml(x=True, y=True, z=True),
-)
 
-
-def run_emulated(simulation: Simulation, task_name: str = None) -> SimulationData:
+def run_emulated(simulation: Simulation, **kwargs) -> SimulationData:
     """Emulates a simulation run."""
 
+    from scipy.ndimage.filters import gaussian_filter
+
     def make_data(coords: dict, data_array_type: type, is_complex: bool = False) -> "data_type":
         """make a random DataArray out of supplied coordinates and data_type."""
         data_shape = [len(coords[k]) for k in data_array_type._dims]
         data = np.random.random(data_shape)
         data = (1 + 1j) * data if is_complex else data
-        return data_array_type(data, coords=coords)
+        data = gaussian_filter(data, sigma=0.5)  # smooth out the data a little so it isnt random
+        data_array = data_array_type(data, coords=coords)
+        return data_array
 
     def make_field_data(monitor: FieldMonitor) -> FieldData:
         """make a random FieldData from a FieldMonitor."""
@@ -340,6 +320,25 @@ def run_emulated(simulation: Simulation, task_name: str = None) -> SimulationDat
             **field_cmps
         )
 
+    def make_eps_data(monitor: PermittivityMonitor) -> PermittivityData:
+        """make a random PermittivityData from a PermittivityMonitor."""
+        field_mnt = FieldMonitor(**monitor.dict(exclude={"type", "fields"}))
+        field_data = make_field_data(monitor=field_mnt)
+        return PermittivityData(
+            monitor=monitor, eps_xx=field_data.Ex, eps_yy=field_data.Ey, eps_zz=field_data.Ez
+        )
+
+    def make_diff_data(monitor: DiffractionMonitor) -> DiffractionData:
+        """make a random PermittivityData from a PermittivityMonitor."""
+        f = list(monitor.freqs)
+        orders_x = np.linspace(-1, 1, 3)
+        orders_y = np.linspace(-2, 2, 5)
+        coords = dict(orders_x=orders_x, orders_y=orders_y, f=f)
+        values = np.random.random((len(orders_x), len(orders_y), len(f)))
+        data = DiffractionDataArray(values, coords=coords)
+        field_data = {field: data for field in ("Er", "Etheta", "Ephi", "Hr", "Htheta", "Hphi")}
+        return DiffractionData(monitor=monitor, sim_size=(1, 1), bloch_vecs=(0, 0), **field_data)
+
     def make_mode_data(monitor: ModeMonitor) -> ModeData:
         """make a random ModeData from a ModeMonitor."""
         mode_indices = np.arange(monitor.mode_spec.num_modes)
@@ -355,35 +354,69 @@ def run_emulated(simulation: Simulation, task_name: str = None) -> SimulationDat
         amps = make_data(coords=coords_amps, data_array_type=ModeAmpsDataArray, is_complex=True)
         return ModeData(monitor=monitor, n_complex=n_complex, amps=amps)
 
-    MONITOR_MAKER_MAP = {FieldMonitor: make_field_data, ModeMonitor: make_mode_data}
+    MONITOR_MAKER_MAP = {
+        FieldMonitor: make_field_data,
+        ModeMonitor: make_mode_data,
+        PermittivityMonitor: make_eps_data,
+        DiffractionMonitor: make_diff_data,
+    }
 
     data = [MONITOR_MAKER_MAP[type(mnt)](mnt) for mnt in simulation.monitors]
 
     return SimulationData(simulation=simulation, data=data)
 
 
-def assert_log_level(caplog, log_level_expected):
+def run_async_emulated(simulations: Dict[str, Simulation], **kwargs) -> BatchData:
+    """Emulate an async run function."""
+    return {task_name: run_emulated(sim) for task_name, sim in simulations.items()}
+
+
+# Log handler used to store log records during tests
+class CaptureHandler:
+    def __init__(self):
+        self.level = 0
+        self.records = []
+
+    def handle(self, level, level_name, message):
+        self.records.append((level, message))
+
+
+# Fixture that captures log records and mek them available as a list of tuples with
+# the log level and message
+@pytest.fixture
+def log_capture(monkeypatch):
+    log_capture = CaptureHandler()
+    monkeypatch.setitem(td.log.handlers, "pytest_capture", log_capture)
+    return log_capture.records
+
+
+def assert_log_level(records, log_level_expected: str):
     """ensure something got logged if log_level is not None.
     note: I put this here rather than utils.py because if we import from utils.py,
     it will validate the sims there and those get included in log.
     """
+    import sys
 
-    # get log output
-    logs = caplog.record_tuples
+    sys.stderr.write(str(records) + "\n")
+
+    if log_level_expected is None:
+        log_level_expected_int = None
+    else:
+        log_level_expected_int = _get_level_int(log_level_expected)
 
     # there's a log but the log level is not None (problem)
-    if logs and not log_level_expected:
+    if records and not log_level_expected_int:
         raise Exception
 
     # we expect a log but none is given (problem)
-    if log_level_expected and not logs:
+    if log_level_expected_int and not records:
         raise Exception
 
     # both expected and got log, check the log levels match
-    if logs and log_level_expected:
-        for log in logs:
-            log_level = log[1]
-            if log_level == log_level_expected:
+    if records and log_level_expected:
+        for log in records:
+            log_level = log[0]
+            if log_level == log_level_expected_int:
                 # log level was triggered, exit
                 return
         raise Exception
