@@ -12,8 +12,9 @@ from .types import Complex, Axis, TYPE_TAG_STR
 from .source import GaussianBeam, ModeSource, PlaneWave
 from .medium import Medium
 
-from ..log import log, SetupError, DataError
 from ..constants import EPSILON_0, MU_0, PML_SIGMA
+from ..exceptions import SetupError, DataError
+from ..log import log
 
 
 class BoundaryEdge(ABC, Tidy3dBaseModel):
@@ -731,3 +732,16 @@ class BoundarySpec(Tidy3dBaseModel):
             (self.y.minus, self.y.plus),
             (self.z.minus, self.z.plus),
         ]
+
+    @cached_property
+    def flipped_bloch_vecs(self) -> BoundarySpec:
+        """Return a copy of the instance where all Bloch vectors are multiplied by -1."""
+        bound_dims = dict(x=self.x.copy(), y=self.y.copy(), z=self.z.copy())
+        for dim_key, bound_dim in bound_dims.items():
+            bound_edges = dict(plus=bound_dim.plus.copy(), minus=bound_dim.minus.copy())
+            for edge_key, bound_edge in bound_edges.items():
+                if isinstance(bound_edge, BlochBoundary):
+                    new_bloch_vec = -1 * bound_edge.bloch_vec
+                    bound_edges[edge_key] = bound_edge.copy(update=dict(bloch_vec=new_bloch_vec))
+            bound_dims[dim_key] = bound_edges
+        return self.copy(update=bound_dims)
