@@ -5,6 +5,7 @@ from __future__ import annotations
 from abc import ABC
 from typing import Union, Tuple, Callable, Dict, List
 import warnings
+
 import xarray as xr
 import numpy as np
 import pydantic as pd
@@ -28,8 +29,9 @@ from ..monitor import FieldProjectionKSpaceMonitor, FieldProjectionSurface
 from ..monitor import DiffractionMonitor
 from ..source import SourceTimeType, CustomFieldSource
 from ..medium import Medium, MediumType
-from ...log import SetupError, DataError, Tidy3dNotImplementedError, log
+from ...exceptions import SetupError, DataError, Tidy3dNotImplementedError
 from ...constants import ETA_0, C_0, MICROMETER
+from ...log import log
 
 
 class MonitorData(Dataset, ABC):
@@ -871,14 +873,14 @@ class FluxTimeData(MonitorData):
     flux: FluxTimeDataArray
 
 
-PROJFIELDTYPE = Union[
+ProjFieldType = Union[
     FieldProjectionAngleDataArray,
     FieldProjectionCartesianDataArray,
     FieldProjectionKSpaceDataArray,
     DiffractionDataArray,
 ]
 
-PROJMONTYPE = Union[
+ProjMonitorType = Union[
     FieldProjectionAngleMonitor,
     FieldProjectionCartesianMonitor,
     FieldProjectionKSpaceMonitor,
@@ -889,38 +891,38 @@ PROJMONTYPE = Union[
 class AbstractFieldProjectionData(MonitorData):
     """Collection of projected fields in spherical coordinates in the frequency domain."""
 
-    monitor: PROJMONTYPE = pd.Field(
+    monitor: ProjMonitorType = pd.Field(
         ...,
         title="Projection monitor",
         description="Field projection monitor.",
     )
 
-    Er: PROJFIELDTYPE = pd.Field(
+    Er: ProjFieldType = pd.Field(
         ...,
         title="Ephi",
         description="Spatial distribution of r-component of the electric field.",
     )
-    Etheta: PROJFIELDTYPE = pd.Field(
+    Etheta: ProjFieldType = pd.Field(
         ...,
         title="Etheta",
         description="Spatial distribution of the theta-component of the electric field.",
     )
-    Ephi: PROJFIELDTYPE = pd.Field(
+    Ephi: ProjFieldType = pd.Field(
         ...,
         title="Ephi",
         description="Spatial distribution of phi-component of the electric field.",
     )
-    Hr: PROJFIELDTYPE = pd.Field(
+    Hr: ProjFieldType = pd.Field(
         ...,
         title="Hphi",
         description="Spatial distribution of r-component of the magnetic field.",
     )
-    Htheta: PROJFIELDTYPE = pd.Field(
+    Htheta: ProjFieldType = pd.Field(
         ...,
         title="Htheta",
         description="Spatial distribution of theta-component of the magnetic field.",
     )
-    Hphi: PROJFIELDTYPE = pd.Field(
+    Hphi: ProjFieldType = pd.Field(
         ...,
         title="Hphi",
         description="Spatial distribution of phi-component of the magnetic field.",
@@ -1661,6 +1663,10 @@ class DiffractionData(AbstractFieldProjectionData):
         the power carried by the wave of that order and polarization equals ``abs(amps)^2``.
         """
         cos_theta = np.cos(np.nan_to_num(self.angles[0]))
+
+        # will set amplitudes to 0 for glancing or negative angles
+        cos_theta[cos_theta <= 0] = np.inf
+
         norm = 1.0 / np.sqrt(2.0 * self.eta) / np.sqrt(cos_theta)
         amp_theta = self.Etheta.values * norm
         amp_phi = self.Ephi.values * norm
