@@ -12,7 +12,9 @@ from ..simulation import Simulation
 from ..boundary import BlochBoundary
 from ..types import Ax, Axis, annotate_type, Literal
 from ..viz import equal_aspect, add_ax_if_none
-from ...log import DataError, log, Tidy3dKeyError, ValidationError
+from ...exceptions import DataError, Tidy3dKeyError, ValidationError
+from ...log import log
+
 
 DATA_TYPE_MAP = {data.__fields__["monitor"].type_: data for data in MonitorDataTypes}
 
@@ -53,7 +55,8 @@ class SimulationData(Tidy3dBaseModel):
     data: Tuple[annotate_type(MonitorDataType), ...] = pd.Field(
         ...,
         title="Monitor Data",
-        description="Mapping of monitor name to :class:`.MonitorData` instance.",
+        description="List of :class:`.MonitorData` instances "
+        "associated with the monitors of the original :class:`.Simulation`.",
     )
 
     log: str = pd.Field(
