@@ -17,7 +17,7 @@ from shapely.errors import ShapelyDeprecationWarning
 from ..base import Tidy3dBaseModel
 from ..types import Axis, ArrayLike
 from ..structure import Structure, MeshOverrideStructure, StructureType
-from ...log import SetupError, ValidationError
+from ...exceptions import SetupError, ValidationError
 from ...constants import C_0, fp_eps
 
 _ROOTS_TOL = 1e-10
@@ -149,7 +149,10 @@ class GradedMesher(Mesher):
             bbox_2d = shapely_box(bbox[0, 0], bbox[0, 1], bbox[1, 0], bbox[1, 1])
 
             # List of structure indexes that may intersect the current structure in 2D
-            query_inds = tree.query_items(bbox_2d)
+            try:
+                query_inds = tree.query_items(bbox_2d)
+            except AttributeError:
+                query_inds = tree.query(bbox_2d)
 
             # Remove all lower structures that the current structure completely contains
             inds_lower = [
