@@ -11,9 +11,9 @@ from typing_extensions import Annotated
 
 import pydantic
 import numpy as np
-from matplotlib.axes._subplots import Axes
+from matplotlib.axes import Axes
 from shapely.geometry.base import BaseGeometry
-from ..log import ValidationError
+from ..exceptions import ValidationError
 
 """ Numpy Arrays """
 
@@ -152,6 +152,9 @@ PlanePosition = Literal["bottom", "middle", "top"]
 
 """ medium """
 
+# custom medium
+InterpMethod = Literal["nearest", "linear"]
+
 # Complex = Union[complex, ComplexNumber]
 Complex = Union[tidycomplex, ComplexNumber]
 PoleAndResidue = Tuple[Complex, Complex]
