@@ -1,9 +1,11 @@
 # pylint:disable=unused-argument
 """ Defines various validation functions that get used to ensure inputs are legit """
+from typing import Any
+
 import pydantic
 
 from .geometry import Box
-from ..log import ValidationError, SetupError
+from ..exceptions import ValidationError, SetupError
 
 """ Explanation of pydantic validators:
 
@@ -38,6 +40,14 @@ from ..log import ValidationError, SetupError
 """
 
 
+def get_value(key: str, values: dict) -> Any:
+    """Grab value from values dictionary. If not present, raise an error before continuing."""
+    val = values.get(key)
+    if val is None:
+        raise ValidationError(f"value {key} not defined, must be present to validate.")
+    return val
+
+
 def assert_plane():
     """makes sure a field's `size` attribute has exactly 1 zero"""
 
@@ -153,7 +163,22 @@ def enforce_monitor_fields_present():
     def _contains_fields(cls, values):
         """Make sure the initially specified fields are here."""
         for field_name in values.get("monitor").fields:
-            assert values.get(field_name) is not None, f"missing field {field_name}"
+            if values.get(field_name) is None:
+                raise SetupError(f"missing field {field_name}")
         return values
 
     return _contains_fields
+
+
+def required_if_symmetry_present(field_name: str):
+    """Make a field required (not None) if any non-zero symmetry eigenvalue is present."""
+
+    @pydantic.validator(field_name, allow_reuse=True, always=True)
+    def _make_required(cls, val, values):
+        """Ensure val is not None if the symmetry is non-zero along any dimension."""
+        symmetry = values.get("symmetry")
+        if any(sym_val != 0 for sym_val in symmetry) and val is None:
+            raise SetupError(f"'{field_name}' must be provided if symmetry present.")
+        return val
+
+    return _make_required
