@@ -4,14 +4,16 @@ from __future__ import annotations
 from typing import Dict, Callable
 import json
 import functools
-import yaml
 
+import yaml
 import pydantic as pd
 
+from .log import log
 from .version import __version__
-from .log import FileError, SetupError, log
+from .exceptions import FileError, SetupError
 from .components.base import Tidy3dBaseModel
 
+
 """Storing version numbers."""
 
 
@@ -205,6 +207,19 @@ def iterate_update_dict(update_dict: Dict, update_types: Dict[str, Callable]):
 @updates_from_version("1.8")
 def update_1_8(sim_dict: dict) -> dict:
     """Updates version 1.8."""
+
+    def fix_missing_scalar_field(mnt_dict: dict) -> dict:
+        for key, val in mnt_dict["field_dataset"].items():
+            if val == "XR.DATAARRAY":
+                mnt_dict["field_dataset"][key] = "ScalarFieldDataArray"
+        return mnt_dict
+
+    iterate_update_dict(
+        update_dict=sim_dict,
+        update_types={
+            "CustomFieldSource": fix_missing_scalar_field,
+        },
+    )
     return sim_dict
 
 
