@@ -1,7 +1,6 @@
 """Provides lowest level, user-facing interface to server."""
 
 import json
-import logging
 import os
 import time
 from datetime import datetime, timedelta
@@ -19,12 +18,14 @@ from .s3utils import upload_string, get_s3_sts_token, download_file
 
 from .s3utils import upload_file
 from .task import TaskId, TaskInfo, Folder
+from ..log import log
 from ..components.data.sim_data import SimulationData
 from ..components.simulation import Simulation
 from ..components.types import Literal
-from ..log import log, WebError
+from ..exceptions import WebError
 from ..version import __version__
 
+
 REFRESH_TIME = 0.3
 TOTAL_DOTS = 3
 # file names when uploading to S3
@@ -38,6 +39,7 @@ def run(  # pylint:disable=too-many-arguments
     folder_name: str = "default",
     path: str = "simulation_data.hdf5",
     callback_url: str = None,
+    verbose: bool = True,
 ) -> SimulationData:
     """Submits a :class:`.Simulation` to server, starts running, monitors progress, downloads,
     and loads results as a :class:`.SimulationData` object.
@@ -55,6 +57,8 @@ def run(  # pylint:disable=too-many-arguments
     callback_url : str = None
         Http PUT url to receive simulation finish event. The body content is a json file with
         fields ``{'id', 'status', 'name', 'workUnit', 'solverVersion'}``.
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
 
     Returns
     -------
@@ -66,14 +70,19 @@ def run(  # pylint:disable=too-many-arguments
         task_name=task_name,
         folder_name=folder_name,
         callback_url=callback_url,
+        verbose=verbose,
     )
     start(task_id)
-    monitor(task_id)
-    return load(task_id=task_id, path=path)
+    monitor(task_id, verbose=verbose)
+    return load(task_id=task_id, path=path, verbose=verbose)
 
 
 def upload(  # pylint:disable=too-many-locals,too-many-arguments
-    simulation: Simulation, task_name: str, folder_name: str = "default", callback_url: str = None
+    simulation: Simulation,
+    task_name: str,
+    folder_name: str = "default",
+    callback_url: str = None,
+    verbose: bool = True,
 ) -> TaskId:
     """Upload simulation to server, but do not start running :class:`.Simulation`.
 
@@ -88,6 +97,8 @@ def upload(  # pylint:disable=too-many-locals,too-many-arguments
     callback_url : str = None
         Http PUT url to receive simulation finish event. The body content is a json file with
         fields ``{'id', 'status', 'name', 'workUnit', 'solverVersion'}``.
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
 
     Returns
     -------
@@ -117,10 +128,11 @@ def upload(  # pylint:disable=too-many-locals,too-many-arguments
         error_json = json.loads(e.response.text)
         raise WebError(error_json["error"]) from e
     # log the task_id so users can copy and paste it from STDOUT / file if the need it later.
-    log.info(f"Created task '{task_name}' with task_id '{task_id}'.")
+    if verbose:
+        log.info(f"Created task '{task_name}' with task_id '{task_id}'.")
 
     # pylint:disable=protected-access
-    upload_string(task_id, simulation._json_string, SIM_FILE_JSON)
+    upload_string(task_id, simulation._json_string, SIM_FILE_JSON, verbose=verbose)
     if len(simulation.custom_datasets) > 0:
         # Also upload hdf5 containing all data.
         # The temp file will be re-opened in `to_hdf5` which can cause an error on some systems
@@ -128,7 +140,7 @@ def upload(  # pylint:disable=too-many-locals,too-many-arguments
         data_file = tempfile.NamedTemporaryFile()  # pylint:disable=consider-using-with
         data_file.close()
         simulation.to_hdf5(data_file.name)
-        upload_file(task_id, data_file.name, SIM_FILE_HDF5)
+        upload_file(task_id, data_file.name, SIM_FILE_HDF5, verbose=verbose)
 
     # log the url for the task in the web UI
     log.debug(f"{DEFAULT_CONFIG.website_endpoint}/folders/{folder.projectId}/tasks/{task_id}")
@@ -166,6 +178,9 @@ def start(task_id: TaskId) -> None:
         Unique identifier of task on server.  Returned by :meth:`upload`.
     solver_version : str
         Supply or override a specific solver version to the task.
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
+
     Note
     ----
     To monitor progress, can call :meth:`monitor` after starting simulation.
@@ -213,14 +228,17 @@ def get_run_info(task_id: TaskId):
         return None, None
 
 
-# pylint: disable=too-many-statements
-def monitor(task_id: TaskId) -> None:
+# pylint: disable=too-many-statements, too-many-locals, too-many-branches
+def monitor(task_id: TaskId, verbose: bool = True) -> None:
+    # pylint:disable=too-many-statements
     """Print the real time task progress until completion.
 
     Parameters
     ----------
     task_id : str
         Unique identifier of task on server.  Returned by :meth:`upload`.
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
 
     Note
     ----
@@ -230,87 +248,114 @@ def monitor(task_id: TaskId) -> None:
     task_info = get_info(task_id)
     task_name = task_info.taskName
 
-    status = None
-
     break_statuses = ("success", "error", "diverged", "deleted", "draft")
 
-    show_cost = True
-
-    def get_status(show_cost=False):
-        """Get status for this task (called many times below, so put into function).
-        If the estimated cost is available, it is displayed if requested."""
+    def get_status() -> str:
+        """Get status for this task."""
         task_info = get_info(task_id)
         status = task_info.status
         if status == "visualize":
             return "success"
         if status == "error":
             raise WebError("Error running task!")
+        return status
 
-        # log the maximum flex unit cost
-        est_flex_unit = task_info.estFlexUnit
-        if show_cost is True and est_flex_unit is not None and est_flex_unit > 0:
-            log.info(f"Maximum FlexUnit cost: {est_flex_unit:1.3f}")
-            # Set show_cost to False so that it is only shown once during the run
-            show_cost = False
+    def get_estimated_cost() -> float:
+        """Get estimated cost, if None, is not ready."""
+        task_info = get_info(task_id)
+        return task_info.estFlexUnit
 
-        return status, show_cost
+    def monitor_preprocess() -> None:
+        """Periodically check the status."""
+        status = get_status()
+        while status not in break_statuses and status != "running":
+            new_status = get_status()
+            if new_status != status:
+                status = new_status
+                if verbose and status != "running":
+                    log.info(f"status = {status}")
+            time.sleep(REFRESH_TIME)
 
-    console = Console()
+    status = get_status()
+
+    if verbose:
+        console = Console()
+        log.info(f"status = {status}")
 
-    status, show_cost = get_status(show_cost=show_cost)
-    log.info(f"status = {status}")
     # already done
     if status in break_statuses:
         return
 
     # preprocessing
-    with console.status(f"[bold green]Starting '{task_name}'...", spinner="runner"):
-        while status not in break_statuses and status != "running":
-            new_status, show_cost = get_status(show_cost=show_cost)
-            if new_status != status:
-                status = new_status
-                if status != "running":
-                    log.info(f"status = {status}")
-            time.sleep(REFRESH_TIME)
+    if verbose:
+        with console.status(f"[bold green]Starting '{task_name}'...", spinner="runner"):
+            monitor_preprocess()
+    else:
+        monitor_preprocess()
 
-    # startup phase where run info is not available
-    log.info("starting up solver")
-    while get_run_info(task_id)[0] is None and status == "running":
-        status, show_cost = get_status(show_cost=show_cost)
+    # if the estimated cost is ready, print it
+    if verbose:
+        est_flex_unit = get_estimated_cost()
+        if est_flex_unit is not None and est_flex_unit > 0:
+            log.info(
+                f"Maximum FlexUnit cost: {est_flex_unit:1.3f}. Use 'web.real_cost(task_id)' to "
+                "get the billed FlexUnit cost after a simulation run."
+            )
+        log.info("starting up solver")
+
+    # while running but before the percentage done is available, keep waiting
+    while get_run_info(task_id)[0] is None and get_status() == "running":
         time.sleep(REFRESH_TIME)
 
-    # phase where run % info is available
-    log.info("running solver")
-    with Progress(console=console) as progress:
-        pbar_pd = progress.add_task("% done", total=100)
-        perc_done, _ = get_run_info(task_id)
-        while perc_done is not None and perc_done < 100 and status == "running":
-            status, show_cost = get_status(show_cost=show_cost)
-            perc_done, field_decay = get_run_info(task_id)
-            new_description = f"% done (field decay = {field_decay:.2e})"
-            progress.update(pbar_pd, completed=perc_done, description=new_description)
-            time.sleep(1.0)
-        if perc_done is not None and perc_done < 100:
-            log.info("early shutoff detected, exiting.")
-        else:
-            progress.update(pbar_pd, completed=100)
+    # while running but percentage done is available
+    if verbose:
 
-    # postprocessing
-    if status != "running":
-        log.info(f"status = {status}")
-    with console.status(f"[bold green]Finishing '{task_name}'...", spinner="runner"):
-        while status not in break_statuses:
-            new_status, show_cost = get_status(show_cost=show_cost)
-            if new_status != status:
-                status = new_status
-                log.info(f"status = {status}")
+        # verbose case, update progressbar
+        log.info("running solver")
+        with Progress(console=console) as progress:
+
+            pbar_pd = progress.add_task("% done", total=100)
+            perc_done, _ = get_run_info(task_id)
+
+            while perc_done is not None and perc_done < 100 and get_status() == "running":
+                perc_done, field_decay = get_run_info(task_id)
+                new_description = f"% done (field decay = {field_decay:.2e})"
+                progress.update(pbar_pd, completed=perc_done, description=new_description)
+                time.sleep(1.0)
+
+            if perc_done is not None and perc_done < 100:
+                log.info("early shutoff detected, exiting.")
+
+            progress.update(pbar_pd, completed=100, refresh=True)
+
+    else:
+
+        # non-verbose case, just keep checking until status is not running or perc_done >= 100
+        perc_done, _ = get_run_info(task_id)
+        while perc_done is not None and perc_done < 100 and get_status() == "running":
+            perc_done, field_decay = get_run_info(task_id)
+            time.sleep(1.0)
+
+    # post processing
+    if verbose:
+
+        status = get_status()
+        if status != "running":
+            log.info(f"status = {status}")
+
+        with console.status(f"[bold green]Finishing '{task_name}'...", spinner="runner"):
+            while status not in break_statuses:
+                new_status = get_status()
+                if new_status != status:
+                    status = new_status
+                    log.info(f"status = {status}")
+                time.sleep(REFRESH_TIME)
+    else:
+        while get_status() not in break_statuses:
             time.sleep(REFRESH_TIME)
 
-    # show final billed cost
-    log.info(f"Billed FlexUnit cost: {get_info(task_id).realFlexUnit:1.3f}")
 
-
-def download(task_id: TaskId, path: str = "simulation_data.hdf5") -> None:
+def download(task_id: TaskId, path: str = "simulation_data.hdf5", verbose: bool = True) -> None:
     """Download results of task and log to file.
 
     Parameters
@@ -319,12 +364,14 @@ def download(task_id: TaskId, path: str = "simulation_data.hdf5") -> None:
         Unique identifier of task on server.  Returned by :meth:`upload`.
     path : str = "simulation_data.hdf5"
         Download path to .hdf5 data file (including filename).
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
 
     """
-    _download_file(task_id, fname="output/monitor_data.hdf5", path=path)
+    _download_file(task_id, fname="output/monitor_data.hdf5", path=path, verbose=verbose)
 
 
-def download_json(task_id: TaskId, path: str = SIM_FILE_JSON) -> None:
+def download_json(task_id: TaskId, path: str = SIM_FILE_JSON, verbose: bool = True) -> None:
     """Download the `.json` file associated with the :class:`.Simulation` of a given task.
 
     Parameters
@@ -333,11 +380,14 @@ def download_json(task_id: TaskId, path: str = SIM_FILE_JSON) -> None:
         Unique identifier of task on server.  Returned by :meth:`upload`.
     path : str = "simulation.json"
         Download path to .json file of simulation (including filename).
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
+
     """
-    _download_file(task_id, fname=SIM_FILE_JSON, path=path)
+    _download_file(task_id, fname=SIM_FILE_JSON, path=path, verbose=verbose)
 
 
-def download_hdf5(task_id: TaskId, path: str = SIM_FILE_HDF5) -> None:
+def download_hdf5(task_id: TaskId, path: str = SIM_FILE_HDF5, verbose: bool = True) -> None:
     """Download the `.hdf5` file associated with the :class:`.Simulation` of a given task.
 
     Parameters
@@ -346,11 +396,14 @@ def download_hdf5(task_id: TaskId, path: str = SIM_FILE_HDF5) -> None:
         Unique identifier of task on server.  Returned by :meth:`upload`.
     path : str = "simulation.hdf5"
         Download path to .hdf5 file of simulation (including filename).
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
+
     """
-    _download_file(task_id, fname=SIM_FILE_HDF5, path=path)
+    _download_file(task_id, fname=SIM_FILE_HDF5, path=path, verbose=verbose)
 
 
-def load_simulation(task_id: TaskId, path: str = SIM_FILE_JSON) -> Simulation:
+def load_simulation(task_id: TaskId, path: str = SIM_FILE_JSON, verbose: bool = True) -> Simulation:
     """Download the `.json` file of a task and load the associated :class:`.Simulation`.
 
     Parameters
@@ -359,17 +412,19 @@ def load_simulation(task_id: TaskId, path: str = SIM_FILE_JSON) -> Simulation:
         Unique identifier of task on server.  Returned by :meth:`upload`.
     path : str = "simulation.json"
         Download path to .json file of simulation (including filename).
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
 
     Returns
     -------
     :class:`.Simulation`
         Simulation loaded from downloaded json file.
     """
-    download_json(task_id, path=path)
+    download_json(task_id, path=path, verbose=verbose)
     return Simulation.from_file(path)
 
 
-def download_log(task_id: TaskId, path: str = "tidy3d.log") -> None:
+def download_log(task_id: TaskId, path: str = "tidy3d.log", verbose: bool = True) -> None:
     """Download the tidy3d log file associated with a task.
 
     Parameters
@@ -383,13 +438,14 @@ def download_log(task_id: TaskId, path: str = "tidy3d.log") -> None:
     ----
     To load downloaded results into data, call :meth:`load` with option `replace_existing=False`.
     """
-    _download_file(task_id, fname="output/tidy3d.log", path=path)
+    _download_file(task_id, fname="output/tidy3d.log", path=path, verbose=verbose)
 
 
 def load(
     task_id: TaskId,
     path: str = "simulation_data.hdf5",
     replace_existing: bool = True,
+    verbose: bool = True,
 ) -> SimulationData:
     """Download and Load simultion results into :class:`.SimulationData` object.
 
@@ -401,6 +457,8 @@ def load(
         Download path to .hdf5 data file (including filename).
     replace_existing: bool = True
         Downloads the data even if path exists (overwriting the existing).
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
 
     Returns
     -------
@@ -409,9 +467,11 @@ def load(
     """
 
     if not os.path.exists(path) or replace_existing:
-        download(task_id=task_id, path=path)
+        download(task_id=task_id, path=path, verbose=verbose)
+
+    if verbose:
+        log.info(f"loading SimulationData from {path}")
 
-    log.info(f"loading SimulationData from {path}")
     sim_data = SimulationData.from_file(path)
 
     final_decay_value = sim_data.final_decay_value
@@ -506,7 +566,7 @@ def get_tasks(
             store_dict["task_name"].append(task["taskName"])
             store_dict["task_id"].append(task["taskId"])
         except KeyError:
-            logging.warning(f"Error with task {task['taskId']}, skipping.")
+            log.warning(f"Error with task {task['taskId']}, skipping.")
 
     sort_inds = sorted(
         range(len(store_dict["submit_time"])),
@@ -549,6 +609,24 @@ def estimate_cost(task_id: str) -> float:
     return resp.get("flex_unit")
 
 
+def real_cost(task_id: str) -> float:
+    """Get the billed cost for given task after it has been run.
+
+    Note
+    ----
+        The billed cost may not be immediately available when the task status is set to ``success``,
+        but should be available shortly after.
+    """
+
+    flex_unit = get_info(task_id).realFlexUnit
+    if not flex_unit:
+        log.warning(
+            f"Billed FlexUnit for task '{task_id}'' is not available. If the task has been "
+            "successfully run, it should be available shortly."
+        )
+    return flex_unit
+
+
 def _query_or_create_folder(folder_name) -> Folder:
     log.debug("query folder")
     method = f"tidy3d/project?projectName={folder_name}"
@@ -568,7 +646,7 @@ def _query_or_create_folder(folder_name) -> Folder:
     return folder
 
 
-def _download_file(task_id: TaskId, fname: str, path: str) -> None:
+def _download_file(task_id: TaskId, fname: str, path: str, verbose: bool = True) -> None:
     """Download a specific file from server.
 
     Parameters
@@ -579,6 +657,9 @@ def _download_file(task_id: TaskId, fname: str, path: str) -> None:
         Name of the file on server (eg. ``monitor_data.hdf5``, ``tidy3d.log``, ``simulation.json``)
     path : str
         Path where the file will be downloaded to (including filename).
+    verbose : bool = True
+        If `True`, will print progressbars and status, otherwise, will run silently.
+
     """
 
     task_info = get_info(task_id)
@@ -589,10 +670,11 @@ def _download_file(task_id: TaskId, fname: str, path: str) -> None:
     if directory != "":
         os.makedirs(directory, exist_ok=True)
 
-    log.info(f'downloading file "{fname}" to "{path}"')
+    if verbose:
+        log.info(f'downloading file "{fname}" to "{path}"')
 
     try:
-        download_file(task_id, fname, path)
+        download_file(task_id, fname, path, verbose=verbose)
     except WebError as e:
         raise e
     except Exception as e:  # pylint:disable=broad-except
